#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Tests for `atenvironment` package."""


import unittest
import os
import random
import string
import itertools

from atenvironment import environment, UnknownKeyword, EnvironMiss, DecoratorSyntaxError


class TestAtenvironment(unittest.TestCase):
    """Tests for `atenvironment` package."""

    def test_decorator(self):
        """Test decorator."""
        key = next(iter(os.environ.keys()))

        @environment(key)
        def test(key):
            return key

        self.assertEqual(test(), os.environ[key])

    def test_noKey(self):
        """Test decorator with no key in environment."""
        for _ in range(7):
            # key = ''.join(random.choices(string.ascii_uppercase, k=random.randrange(42)))
            key = ''.join(random.choice(string.ascii_uppercase) for _ in range(random.randrange(42)))
            if key not in os.environ:
                break

        self.assertFalse(key in os.environ)

        @environment(key)
        def test(key):
            return key

        self.assertRaises(EnvironMiss, test)

    def test_multiple(self):

        keys = tuple(itertools.islice(os.environ, 3))

        @environment(*keys)
        def test(a, b, c):
            return (a, b, c)

        self.assertEqual(tuple([os.environ[key] for key in keys]), test())

        keys = list(keys)
        @environment(*keys)
        def test(a, b, c):
            return (a, b, c)

        self.assertEqual(tuple([os.environ[key] for key in keys]), test())

    def test_multiple_miss(self):

        self.assertTrue('foobar' not in os.environ)
        keys = list(itertools.islice(os.environ, 3))
        keys.append('foobar')

        @environment(*keys)
        def test(a, b, c, d):
            return [a, b, c, d]

        self.assertRaises(EnvironMiss, test)

        keys = keys[:-1]
        @environment('foobar', *keys)
        def test(a,b,c,d):
            return [a, b, c, d]

        self.assertRaises(EnvironMiss, test)

    def test_onerror(self):
        self.assertTrue('foobar' not in os.environ)

        class TestResult(BaseException):
            pass

        def err(val):
            self.assertEqual('foobar', val)
            raise TestResult

        @environment('foobar', onerror=err)
        def test(a):
            return a

        self.assertRaises(TestResult, test)

    def test_onerror_multiple(self):
        class BodyExecuted(BaseException):
            pass

        class TestResultA(BaseException):
            pass

        class TestResultB(BaseException):
            pass

        def err0(x):
            raise TestResultA()

        def err1(x):
            raise TestResultB()

        self.assertTrue('foobar' not in os.environ)
        a0 = list(itertools.islice(os.environ, 3))
        a0.append('foobar')

        a1 = tuple(itertools.islice(os.environ, 3))

        @environment(*a0, onerror=err0)
        @environment(*a1, onerror=err1)
        def test(a, b, c, d, e, f):
            raise BodyExecuted()

        self.assertRaises(TestResultA, test)

        @environment(*a1, onerror=err0)
        @environment(*a0, onerror=err1)
        def test(a, b, c, d, e, f):
            raise BodyExecuted()

        self.assertRaises(TestResultB, test)

    def test_unknown_keyword(self):
        try:
            @environment(next(iter(os.environ.keys())), foobar=None)
            def test(a):
                return a
        except UnknownKeyword:
            pass
        else:
            self.assertTrue(False)

    def test_bad_signature(self):
        class BodyExecuted(BaseException):
            pass

        @environment(next(iter(os.environ.keys())))
        def test():
            raise BodyExecuted()

        self.assertRaises(TypeError, test)

    def test_class(self):
        x = next(iter(os.environ.keys()))
        class TestBed(object):
            @environment(x, in_self="foo")
            def __init__(self):
                pass

        t = TestBed()
        self.assertEqual(os.environ[x], t.foo)

    def test_class_only_one_key(self):
        keys = tuple(itertools.islice(os.environ, 3))

        class TestBed(object):
            @environment(*keys, in_self="foo")
            def __init__(self):
                raise BodyExecuted()

        try:
            TestBed()
        except DecoratorSyntaxError:
            pass
        else:
            self.assertTrue(False)

    def test_self_no_class(self):
        a1 = tuple(itertools.islice(os.environ, 1))
        @environment(*a1, in_self="foo")
        def test(a):
            raise BodyExecuted()
        try:
            test('a')
        except DecoratorSyntaxError:
            pass
        else:
            self.assertTrue(False)



# -*- coding: utf-8 -*-

"""Top-level package for @environment."""

__author__ = """Alexandr Mansurov"""
__email__ = 'alex@eghuro.cz'
__version__ = '0.2.0'

import os
import logging
from inspect import signature
from functools import wraps


class UnknownKeyword(BaseException):
    """Exception indicating unknown keyword was provided to @environment
       decorator in **kwargs.
    """
    pass


class EnvironMiss(KeyError):
    """Error indicating key is not present in environment."""
    pass


class DecoratorSyntaxError(BaseException):
    pass


def _missing(value):
    log = logging.getLogger(__name__)
    log.error("Missing environment variable: %s" % (value))
    raise EnvironMiss(value)


_allowed_keywords = ['onerror', 'in_self']


def environment(*value, **kwargs):
    """@environment decorator.

    Arguments:
       value   -- one or more environment tokens requested

       onerror -- optional function to be called if any of the environment
       tokens in value is not present in environment. Such function must take
       one parameter what is a string value of a missing environment token. If
       onerror is not set, error is logged and EnvironMiss exception is raised

       in_self -- optional variable name in case instance property is to be
       initialized. There must be only one value in such case.

    The decorator checks for presence of environment tokens and if successful
    reads their values to the function parameters of the decorated function
    after any called parameters provided.

    Eg. if calling function(a, b, c) that is decorated with @environment('X')
    the function must be defined as def function(a, b, c, x) and X from the
    environment is read as last parameter.

    When combining decorators or using multiple environment tokens in one
    @environment('X', 'Y', 'Z') the values are loaded from the left to the
    right, from the top to the bottom.

    If a function parameter for @environment is missing, when such function
    is called a TypeError is raised by the interpreter.
    """

    err = _missing

    if kwargs is not None:
        for k in kwargs:
            if k not in _allowed_keywords:
                raise UnknownKeyword(k)

        if 'onerror' in kwargs:
            err = kwargs['onerror']

    def environ_decorator(func):

        @wraps(func)
        def inner(*args):
            for v in value:
                if v not in os.environ:
                    err(v)
            if 'in_self' in kwargs:
                if len(value) != 1:
                    raise DecoratorSyntaxError(value)
                if 'self' not in signature(func).parameters:
                    raise DecoratorSyntaxError()
                args[0].__dict__[kwargs['in_self']] = os.environ[v]
                return func(*args)
            return func(*(list(args) + [os.environ[v] for v in value]))
        return inner
    return environ_decorator



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""The setup script."""

from setuptools import setup, find_packages

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [ ]

setup_requirements = [ ]

test_requirements = [ ]

setup(
    author="Alexandr Mansurov",
    author_email='alex@eghuro.cz',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System',
        'Topic :: Utilities'
    ],
    description="Decorator for convenient loading of environment variables",
    install_requires=requirements,
    license="MIT license",
    long_description=readme + '\n\n' + history,
    include_package_data=True,
    keywords='atenvironment',
    name='atenvironment',
    packages=find_packages(include=['atenvironment']),
    setup_requires=setup_requirements,
    test_suite='tests',
    tests_require=test_requirements,
    url='https://github.com/eghuro/atenvironment',
    version='0.2.0',
    zip_safe=False,
)
